
module TypesEsp where

import General
import Invariant

-- parameter types for Spanish morphology   
   
data Gender = Masc | Fem   
  deriving (Eq, Ord, Show, Read, Enum, Bounded)   
   
data Number = Sg | Pl   
  deriving (Eq, Ord, Show, Read, Enum, Bounded) 

data Eval = Reg | Dim | Aug | Pej    
  deriving (Eq, Ord, Show, Read, Enum, Bounded)  
  
data Person = P1 | P2 | P3   
  deriving (Eq, Ord, Show, Read, Enum, Bounded)   
   
data Case = Sub | PrepExpr | DirObj | IndirObj   
  deriving (Eq, Ord, Show, Read, Enum, Bounded)   
  
data Stress = Unstressed | Stressed   
  deriving (Eq, Ord, Show, Read, Enum, Bounded)   
    
data Grade = Posit | Compar | Superl   
  deriving (Eq, Ord, Show, Read, Enum, Bounded)   
   
data Mood = Ind | Subj   
  deriving (Eq, Ord, Show, Read, Enum, Bounded)  
 
instance Param Gender  where values = enum   
instance Param Number  where values = enum   
instance Param Eval    where values = enum   
instance Param Person  where values = enum   
instance Param Case    where values = enum   
instance Param Stress  where values = enum   
instance Param Grade   where values = enum   
instance Param Mood    where values = enum   
  
   
-- Nouns   
type Noun = NounForm -> Str   
   
data NounForm = NF Number   
  deriving (Eq, Ord, Show, Read)   
   
instance Param NounForm   
  where values = [NF a | a <- values] 

-- Nouns of both genders
type Noun1 = NounForm1 -> Str 

data NounForm1 = NF1 Number Gender Eval   
  deriving (Eq, Ord, Show, Read)    
    
instance Param NounForm1    
  where values = [NF1 a b c | a <- values, b <- values, c <- values]   

--data NounForm1 = NF1 Number Gender  
--  deriving (Eq, Ord, Show, Read)   
   
--instance Param NounForm1   
--  where values = [NF1 a b | a <- values, b <- values]  


-- Adjectives   
type Adjective = AdjForm -> Str   
   
data AdjForm = AF Number Gender   
  deriving (Eq, Ord, Show, Read)   
   
instance Param AdjForm where   
  values = [AF a b | a <- values, b <- values]   

type IrregAdjective = IrregAdjForm -> Str   
   
data IrregAdjForm = IAF Grade Number Gender   
  deriving (Eq, Ord, Show, Read)   
   
instance Param IrregAdjForm where   
  values = [IAF a b c| a <- values, b <- values, c <- values]   
   
-- Verbs   
type Verb = VerbForm -> Str   
   
data VImpers =   
   Inf   
 | Ger   
 | Part    
  deriving (Eq, Ord, Show, Read)   
   
data VPers =   
   Pres Mood Number Person   
 | Past Mood Number Person   
 | Pret      Number Person   
 | Fut  Mood Number Person   
 | Cond      Number Person   
 | Imp       Number Person
 | Pass      Number Gender   
  deriving (Eq, Ord, Show, Read)   
   
data VerbForm =   
   VI VImpers   
 | VP VPers   
  deriving (Eq, Ord, Show, Read)   

instance Param VPers where   
  values = map (uncurries Pres) mnp ++   
	   map (uncurries Past) mnp ++   
           map (uncurry   Pret) np  ++  
           map (uncurries  Fut) mnp ++   
	   map (uncurry   Cond) np  ++   
           map (uncurry    Imp) np  ++ 
	   map (uncurry   Pass) ng   
           where   
             mnp = [(m,n,p) | m <- values, n <- values, p <- values]   
             np  = [(n,p)   | n <- values, p <- values]   
             ng  = [(n,g)   | n <- values, g <- values]   
   
uncurries :: (a -> b -> c -> d) -> ((a,b,c) -> d)   
uncurries f p = f (fst1 p) (snd2 p) (trd3 p) where   
    fst1 (x,_, _) = x   
    snd2 (_,y,_) = y   
    trd3 (_,_,z) = z   
   
instance Param VImpers where   
  values = [Inf] ++ [Ger] ++ [Part]   
   
instance Param VerbForm where   
  values = map VI values ++ map VP values   
  value0 = VI Inf  -- to show the infinitive as dictionary form  
 
----------------------------------------------------------------------  
  
-- closed classes  
 
----------------------------------------------------------------------  
 
-- Articles   
type Article = ArtForm -> Str   

data ArtForm = ArtF Number  
  deriving (Eq, Ord, Show, Read)  

instance Param ArtForm where   
  values = [ArtF a  | a <- values]   

--type Article1 = ArtForm1 -> Str   

--data ArtForm1 = ArtF1 Number AGender   
--  deriving (Eq, Ord, Show, Read)   
   
--instance Param ArtForm1 where   
--  values = [ArtF1 a b | a <- values, b <- values]      

data AGender = Masc1 | Fem1| Neutr   
  deriving (Eq, Ord, Show, Read, Enum, Bounded)   

instance Param AGender where values = enum 
 
--Pronouns 
 
-- Personal pronouns 
type PersPron = PersPronForm -> Str   
   
data PersPronForm = PPF Case Number Person AGender   
  deriving (Eq, Ord, Show, Read)   
   
instance Param PersPronForm where   
  values = [PPF a b c d | a <- values, b <- values, c <- values, d <- values] 
 
-- Reflexive pronouns 
type ReflPron = ReflPronForm -> Str  
  
data ReflPronForm = RPF Number Person  
  deriving (Eq, Ord, Show, Read)  
  
instance Param ReflPronForm where  
  values = [RPF a b | a <- values, b <- values] 
 
-- Possessiv pronouns 
type PossPron = PossPronForm -> Str  
  
data PossPronForm = PossPF Stress Number Person Gender Number                      
  deriving (Eq, Ord, Show, Read)  
  
instance Param PossPronForm where  
  values = [PossPF a b c d e | a <- values, b <- values, c <- values, d <- values, e <- values]   

-- Demonstrative pronouns  
type DemoPron = DemoPronForm -> Str  
  
data DemoPronForm = DPF Number AGender  
  deriving (Eq, Ord, Show, Read)  
  
instance Param DemoPronForm where  
  values = [DPF a b| a <- values, b <- values]  
 
-- Relative pronouns   
type RelPron = RelPronForm -> Str  
  
data RelPronForm = RelPF Number Gender                      
  deriving (Eq, Ord, Show, Read)  
  
instance Param RelPronForm where  
  values = [RelPF a b | a <- values, b <- values] 
 
-- Interrogative pronouns  
type IntPron = IntPronForm -> Str  
  
data IntPronForm = IPF Number Gender                      
  deriving (Eq, Ord, Show, Read)  
  
instance Param IntPronForm where  
  values = [IPF a b | a <- values, b <- values] 

-- Indefinite pronouns  
type IndefPron1 = IndefPronForm1 -> Str  
  
data IndefPronForm1 = UniPF Number Gender  
  deriving (Eq, Ord, Show, Read)  
  
instance Param IndefPronForm1 where  
  values = [UniPF a b | a <- values, b <- values] 


type IndefPron = IndefPronForm -> Str  
  
data IndefPronForm = IndefPronForm
  deriving (Eq, Ord, Show, Read, Enum, Bounded)  
  
instance Param IndefPronForm where  
  values    = enum
  prValue _ = invar

-- Irreg adverbs 

type IrregAdverb = IrregAdvForm -> Str   
   
data IrregAdvForm = IAdvF Grade   
  deriving (Eq, Ord, Show, Read)   
   
instance Param IrregAdvForm where   
  values = [IAdvF a | a <- values]   


type Adverb = AdverbForm -> Str

data AdverbForm = AdverbForm
 deriving (Eq, Ord, Show, Read, Enum, Bounded)

instance Param AdverbForm where
                           values = enum
                           prValue _ = invar


type PN = PNForm -> Str

data PNForm = PNForm
 deriving (Eq, Ord, Show, Read, Enum, Bounded)

instance Param PNForm where
                      values = enum
                      prValue _ = invar


type Prep = PrepForm -> Str

data PrepForm = PrepForm
 deriving (Eq, Ord, Show, Read, Enum, Bounded)

instance Param PrepForm where
                      values = enum
                      prValue _ = invar

type Conj = ConjForm -> Str

data ConjForm = ConjForm
 deriving (Eq, Ord, Show, Read, Enum, Bounded)

instance Param ConjForm where
                      values = enum
                      prValue _ = invar

type Interj = InterjForm -> Str

data InterjForm = InterjForm
 deriving (Eq, Ord, Show, Read, Enum, Bounded)

instance Param InterjForm where
                      values = enum
                      prValue _ = invar

-- Numbers
type Numb = NumForm -> Str  
  
data NumForm = Cardinal Gender  
  deriving (Eq, Ord, Show, Read)  
  
instance Param NumForm where  
  values = [Cardinal a | a <- values]    
   
type Order = OrdForm -> Str  
  
data OrdForm = Ordinal Number Gender  
  deriving (Eq, Ord, Show, Read)  
  
instance Param OrdForm where  
  values = [Ordinal a b | a <- values, b <- values]    
   

